@@ -89,6 +89,12 @@ get_provider_group (GoaProvider *_provider)
   return GOA_PROVIDER_GROUP_BRANDED;
 }
 
+static GoaProviderFeatures
+get_provider_features (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_FEATURE_BRANDED | GOA_PROVIDER_FEATURE_PHOTOS;
+}
+
 static const gchar *
 get_consumer_key (GoaOAuthProvider *provider)
 {
@@ -450,6 +456,7 @@ goa_flickr_provider_class_init (GoaFlickrProviderClass *klass)
   provider_class->get_provider_type     = get_provider_type;
   provider_class->get_provider_name     = get_provider_name;
   provider_class->get_provider_group         = get_provider_group;
+  provider_class->get_provider_features      = get_provider_features;
   provider_class->build_object          = build_object;
   provider_class->show_account          = show_account;
 
