@@ -26,6 +26,7 @@
 #include <rest/oauth-proxy.h>
 #include <json-glib/json-glib.h>
 
+#include "goalogging.h"
 #include "goaprovider.h"
 #include "goaprovider-priv.h"
 #include "goaoauthprovider.h"
@@ -80,6 +81,18 @@ get_provider_name (GoaProvider *_provider,
   return g_strdup (_("Yahoo"));
 }
 
+static GoaProviderGroup
+get_provider_group (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_GROUP_BRANDED;
+}
+
+static GoaProviderFeatures
+get_provider_features (GoaProvider *_provider)
+{
+  return GOA_PROVIDER_FEATURE_BRANDED;
+}
+
 static const gchar *
 get_consumer_key (GoaOAuthProvider *provider)
 {
@@ -116,6 +129,12 @@ get_callback_uri (GoaOAuthProvider *provider)
   return "https://www.gnome.org/goa-1.0/oauth";
 }
 
+static const gchar *
+get_authentication_cookie (GoaOAuthProvider *provider)
+{
+  return "";
+}
+
 /* ---------------------------------------------------------------------------------------------------- */
 
 static gchar *
@@ -126,6 +145,7 @@ get_identity_sync (GoaOAuthProvider  *provider,
                    GCancellable      *cancellable,
                    GError           **error)
 {
+  GError *identity_error;
   RestProxy *proxy;
   RestProxyCall *call;
   JsonParser *parser;
@@ -136,6 +156,8 @@ get_identity_sync (GoaOAuthProvider  *provider,
   gchar *presentation_identity;
 
   ret = NULL;
+
+  identity_error = NULL;
   proxy = NULL;
   call = NULL;
   parser = NULL;
@@ -172,9 +194,16 @@ get_identity_sync (GoaOAuthProvider  *provider,
   if (!json_parser_load_from_data (parser,
                                    rest_proxy_call_get_payload (call),
                                    rest_proxy_call_get_payload_length (call),
-                                   error))
+                                   &identity_error))
     {
-      g_prefix_error (error, _("Error parsing response as JSON: "));
+      goa_warning ("json_parser_load_from_data() failed: %s (%s, %d)",
+                   identity_error->message,
+                   g_quark_to_string (identity_error->domain),
+                   identity_error->code);
+      g_set_error (error,
+                   GOA_ERROR,
+                   GOA_ERROR_FAILED,
+                   _("Could not parse response"));
       goto out;
     }
 
@@ -182,20 +211,22 @@ get_identity_sync (GoaOAuthProvider  *provider,
   json_data_object = json_object_get_object_member (json_object, "guid");
   if (json_data_object == NULL)
     {
+      goa_warning ("Did not find guid in JSON data");
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find guid member in JSON data"));
+                   _("Could not parse response"));
       goto out;
     }
 
   guid = g_strdup (json_object_get_string_member (json_data_object, "value"));
   if (guid == NULL)
     {
+      goa_warning ("Did not find value in JSON data");
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find value member in JSON data"));
+                   _("Could not parse response"));
       goto out;
     }
 
@@ -232,9 +263,16 @@ get_identity_sync (GoaOAuthProvider  *provider,
   if (!json_parser_load_from_data (parser,
                                    rest_proxy_call_get_payload (call),
                                    rest_proxy_call_get_payload_length (call),
-                                   error))
+                                   &identity_error))
     {
-      g_prefix_error (error, _("Error parsing usercard response as JSON: "));
+      goa_warning ("json_parser_load_from_data() failed: %s (%s, %d)",
+                   identity_error->message,
+                   g_quark_to_string (identity_error->domain),
+                   identity_error->code);
+      g_set_error (error,
+                   GOA_ERROR,
+                   GOA_ERROR_FAILED,
+                   _("Could not parse response"));
       goto out;
     }
 
@@ -242,20 +280,22 @@ get_identity_sync (GoaOAuthProvider  *provider,
   json_data_object = json_object_get_object_member (json_object, "profile");
   if (json_data_object == NULL)
     {
+      goa_warning ("Did not find profile in JSON data");
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find profile member in JSON data"));
+                   _("Could not parse response"));
       goto out;
     }
 
   presentation_identity = g_strdup (json_object_get_string_member (json_data_object, "nickname"));
   if (presentation_identity == NULL)
     {
+      goa_warning ("Did not find nickname in JSON data");
       g_set_error (error,
                    GOA_ERROR,
                    GOA_ERROR_FAILED,
-                   _("Didn't find nickname member in JSON data"));
+                   _("Could not parse response"));
       goto out;
     }
 
@@ -268,6 +308,7 @@ get_identity_sync (GoaOAuthProvider  *provider,
     }
 
  out:
+  g_clear_error (&identity_error);
   g_free (presentation_identity);
   g_free (guid);
   if (call != NULL)
@@ -279,6 +320,40 @@ get_identity_sync (GoaOAuthProvider  *provider,
 
 /* ---------------------------------------------------------------------------------------------------- */
 
+static gboolean
+is_deny_node (GoaOAuthProvider *provider, WebKitDOMNode *node)
+{
+  return FALSE;
+}
+
+static gboolean
+is_identity_node (GoaOAuthProvider *provider, WebKitDOMHTMLInputElement *element)
+{
+  return FALSE;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
+static gchar *
+parse_request_token_error (GoaOAuthProvider *provider, RestProxyCall *call)
+{
+  const gchar *payload;
+  gchar *msg;
+  guint status;
+
+  msg = NULL;
+
+  payload = rest_proxy_call_get_payload (call);
+  status = rest_proxy_call_get_status_code (call);
+
+  if (status == 401 && g_strcmp0 (payload, "oauth_problem=timestamp_refused") == 0)
+    msg = g_strdup (_("Your system time is invalid. Check your date and time settings."));
+
+  return msg;
+}
+
+/* ---------------------------------------------------------------------------------------------------- */
+
 static gboolean
 build_object (GoaProvider         *provider,
               GoaObjectSkeleton   *object,
@@ -351,16 +426,22 @@ goa_yahoo_provider_class_init (GoaYahooProviderClass *klass)
   provider_class = GOA_PROVIDER_CLASS (klass);
   provider_class->get_provider_type     = get_provider_type;
   provider_class->get_provider_name     = get_provider_name;
+  provider_class->get_provider_group    = get_provider_group;
+  provider_class->get_provider_features = get_provider_features;
   provider_class->build_object          = build_object;
   provider_class->show_account          = show_account;
 
   oauth_class = GOA_OAUTH_PROVIDER_CLASS (klass);
   oauth_class->get_identity_sync        = get_identity_sync;
+  oauth_class->is_deny_node             = is_deny_node;
+  oauth_class->is_identity_node         = is_identity_node;
   oauth_class->get_consumer_key         = get_consumer_key;
   oauth_class->get_consumer_secret      = get_consumer_secret;
   oauth_class->get_request_uri          = get_request_uri;
   oauth_class->get_authorization_uri    = get_authorization_uri;
   oauth_class->get_token_uri            = get_token_uri;
   oauth_class->get_callback_uri         = get_callback_uri;
+  oauth_class->get_authentication_cookie = get_authentication_cookie;
   oauth_class->get_use_external_browser = get_use_external_browser;
+  oauth_class->parse_request_token_error = parse_request_token_error;
 }
