@@ -2,20 +2,20 @@
 /*
  * Copyright (C) 2012 Red Hat, Inc.
  *
- * This program is free software; you can redistribute it and/or modify
- * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General
+ * Public License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
+ * Boston, MA 02111-1307, USA.
  *
  * Author: Ray Strode
  * Based on work by Colin Walters
@@ -256,27 +256,12 @@ goa_alarm_init (GoaAlarm *self)
   g_rec_mutex_init (&self->priv->lock);
 }
 
-static gboolean
-async_alarm_cancel_idle_cb (gpointer user_data)
-{
-  GoaAlarm *self = user_data;
-
-  clear_scheduled_wakeups (self);
-  return G_SOURCE_REMOVE;
-}
-
 static void
 on_cancelled (GCancellable *cancellable, gpointer user_data)
 {
   GoaAlarm *self = GOA_ALARM (user_data);
-  GSource *idle_source;
 
-
-  idle_source = g_idle_source_new ();
-  g_source_set_priority (idle_source, G_PRIORITY_HIGH_IDLE);
-  g_source_set_callback (idle_source, async_alarm_cancel_idle_cb, g_object_ref (self), g_object_unref);
-  g_source_attach (idle_source, self->priv->context);
-  g_source_unref (idle_source);
+  clear_scheduled_wakeups (self);
 }
 
 static void
@@ -359,52 +344,23 @@ out:
 
 #ifdef HAVE_TIMERFD
 static gboolean
-on_timer_source_ready (GObject *stream, GTask *task)
+on_timer_source_ready (GObject *stream, GoaAlarm *self)
 {
   gint64 number_of_fires;
   gssize bytes_read;
   gboolean run_again = FALSE;
-  GError *error = NULL;
-  GoaAlarm *self;
-  GCancellable *cancellable;
-
-  self = g_task_get_source_object (task);
-  cancellable = g_task_get_cancellable (task);
 
   g_return_val_if_fail (GOA_IS_ALARM (self), FALSE);
+  g_return_val_if_fail (self->priv->type == GOA_ALARM_TYPE_TIMER, FALSE);
 
   g_rec_mutex_lock (&self->priv->lock);
-
-  if (self->priv->type == GOA_ALARM_TYPE_UNSCHEDULED)
-    {
-      goa_debug ("GoaAlarm: timer source was unscheduled after "
-                 "callback was invoked, but before callback got "
-                 "the lock.");
-      goto out;
-    }
-  else if (self->priv->type != GOA_ALARM_TYPE_TIMER)
-    {
-      goa_warning ("GoaAlarm: timer source ready callback called "
-                   "when timer source isn't supposed to be used. "
-                   "Current timer type is %u", self->priv->type);
-      goto out;
-    }
-
-  if (g_cancellable_is_cancelled (cancellable))
+  if (g_cancellable_is_cancelled (self->priv->cancellable))
     goto out;
 
   bytes_read =
     g_pollable_input_stream_read_nonblocking (G_POLLABLE_INPUT_STREAM (stream),
                                               &number_of_fires, sizeof (gint64),
-                                              NULL, &error);
-
-  if (bytes_read < 0)
-    {
-      goa_warning ("GoaAlarm: failed to read from timer fd: %s\n",
-                   error->message);
-      g_error_free (error);
-      goto out;
-    }
+                                              NULL, NULL);
 
   if (bytes_read == sizeof (gint64))
     {
@@ -423,14 +379,9 @@ out:
 }
 
 static void
-clear_timer_source (GTask *task)
+clear_timer_source_pointer (GoaAlarm *self)
 {
-  GoaAlarm *self;
-
-  self = g_task_get_source_object (task);
   self->priv->timer.source = NULL;
-
-  g_object_unref (task);
 }
 #endif
 
@@ -442,7 +393,6 @@ schedule_wakeups_with_timerfd (GoaAlarm *self)
   int fd;
   int result;
   GSource *source;
-  GTask *task;
   static gboolean seen_before = FALSE;
 
   if (!seen_before)
@@ -475,16 +425,14 @@ schedule_wakeups_with_timerfd (GoaAlarm *self)
   self->priv->type = GOA_ALARM_TYPE_TIMER;
   self->priv->timer.stream = g_unix_input_stream_new (fd, TRUE);
 
-  task = g_task_new (self, self->priv->cancellable, NULL, NULL);
-
   source =
     g_pollable_input_stream_create_source (G_POLLABLE_INPUT_STREAM
                                            (self->priv->timer.stream),
                                            self->priv->cancellable);
   self->priv->timer.source = source;
   g_source_set_callback (self->priv->timer.source,
-                         (GSourceFunc) on_timer_source_ready, task,
-                         (GDestroyNotify) clear_timer_source);
+                         (GSourceFunc) on_timer_source_ready, self,
+                         (GDestroyNotify) clear_timer_source_pointer);
   g_source_attach (self->priv->timer.source, self->priv->context);
   g_source_unref (source);
 
@@ -609,7 +557,6 @@ goa_alarm_set_time (GoaAlarm *self, GDateTime *time, GCancellable *cancellable)
   if (g_cancellable_is_cancelled (cancellable))
     return;
 
-  g_rec_mutex_lock (&self->priv->lock);
   if (self->priv->cancellable != NULL && self->priv->cancellable != cancellable)
     g_cancellable_cancel (self->priv->cancellable);
 
@@ -639,12 +586,12 @@ goa_alarm_set_time (GoaAlarm *self, GDateTime *time, GCancellable *cancellable)
 
   self->priv->context = g_main_context_ref (g_main_context_default ());
 
+  g_object_notify (G_OBJECT (self), "time");
+
   schedule_wakeups (self);
 
   /* Wake up right away, in case it's already expired leaving the gate */
   schedule_immediate_wakeup (self);
-  g_rec_mutex_unlock (&self->priv->lock);
-  g_object_notify (G_OBJECT (self), "time");
 }
 
 GDateTime *
